package net.sergeych.crypto2

import kotlinx.serialization.Serializable

@Serializable
sealed class UniversalKey: KeyInstance {

    abstract val keyBytes: UByteArray
    abstract val magick: KeysMagickNumber

    override val id by lazy { KeyId(magick, keyBytes) }

    override fun equals(other: Any?): Boolean {
        return other is UniversalKey && other.keyBytes contentEquals keyBytes
    }

    override fun hashCode(): Int {
        return keyBytes.contentHashCode()
    }

    override fun toString(): String = keyBytes.encodeToBase64Url()

    companion object {
        fun newSecretKey() = Asymmetric.newSecretKey()
        fun newSigningKey() = SigningSecretKey.new()
        @Suppress("unused")
        fun newSymmetricKey() = SymmetricKey.new()

    }
}

open class IllegalSignatureException(text: String = "signed data is tampered or signature is corrupted") :
    IllegalStateException(text)

class ExpiredSignatureException(text: String) : IllegalSignatureException(text)